@@ -3,6 +3,11 @@ author: Mislav Sever
 
 A1000
 A Python module for the ReversingLabs A1000 appliance REST API.
+
+Copyright (c) ReversingLabs International GmbH. 2016-2021
+
+This unpublished material is proprietary to ReversingLabs International GmbH.. All rights reserved.
+Reproduction or distribution, in whole or in part, is forbidden except by express written permission of ReversingLabs International GmbH.
 """
 
 import requests
@@ -60,7 +65,12 @@ class A1000(object):
     def __init__(self, host, username=None, password=None, token=None, fields=__FIELDS, wait_time_seconds=2, retries=10,
                  verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT):
 
-        self._host = self.__validate_host(host)
+        if not isinstance(host, str):
+            raise WrongInputError("host parameter must be string.")
+        if not host.startswith(("http://", "https://")):
+            raise WrongInputError("host parameter must contain a protocol definition at the beginning. "
+                                  "Possible values: 'http://', 'https://'")
+        self._host = host
         self._url = "{host}{{endpoint}}".format(host=self._host)
         self._verify = verify
         self._user_agent = user_agent
@@ -91,24 +101,6 @@ class A1000(object):
             raise WrongInputError("retries must be an integer.")
         self._retries = retries
 
-    @staticmethod
-    def __validate_host(host):
-        """Returns a formatted host URL including the protocol prefix.
-            :param host: URL string
-            :type host: str
-            :returns: formatted URL string
-            :rtype: str
-        """
-        if not isinstance(host, str):
-            raise WrongInputError("host parameter must be string.")
-
-        if not host.startswith(("http://", "https://")):
-            raise WrongInputError("host parameter must contain a protocol definition at the beginning.")
-
-        host = host.rstrip("/")
-
-        return host
-
     def configuration_dump(self):
         """Returns the configuration of the instantiated A1000 object.
             :return: configuration string
